package com.ruoyi.ibs.customerselect.service;

import com.ruoyi.ibs.customerselect.domain.*;
import com.ruoyi.ibs.list.domain.CustManualTagDTO;
import com.ruoyi.ibs.list.domain.TreeNode;

import java.util.List;

public interface IMyCustomerService {

    /**
     * 查询客户基本信息
     *
     * @return 客户基本信息
     */
    public List<CustBaseInfo> selectSysCustomerBasedataList(CustBaseInfo sysCustomerBasedata);

    Long countCustRetailByRegion(String code);

    Long countCustBusinessByRegion(String code);

    Long countCustMerchantByRegion(String code);

    List<Long> countAllByCode(String code, String deptCode);


    /**
     * 我的客户数据新增修改
     *
     * @return 客户基本信息
     */
    public int custInfoUpdate(CustInfoUpdateFromAnchor custInfoUpdateFromAnchor);

    /**
     * 我的客户数据删除
     *
     * @return 客户基本信息
     */
    public int custInfoDelete(CustInfoDeleteFromAnchor custInfoDeleteFromAnchor);

    public List<CustBaseInfo> selectAddress(List<CustBaseInfo> list);

    public CustListSearchVo selectCustomListSearchVo(CustBaseInfo sysCustomerBasedata);



    public List<TreeNode> selectCustEnumByCustId(String custId,String custType);

    public int editCustTag(CustTagDTO custTagDTO);

    MerchantMcspInfo selectmerchantMessage(String custId);

    int editManualTag(CustManualTagDTO custManualTagDTO);

    List<TreeNode> selectmanualListByCustId(String custId, String custType);
}
